@@ -16,12 +16,13 @@ import socket
 import traceback
 # Django
 from django import db
+from django.conf import settings
 from django.utils import timezone
 from django.utils.translation import ugettext_lazy as _
 from multiprocessing import Event, Process, Value, current_process
 
 # Local
-from django_q import tasks
+import django_q.tasks
 from django_q.brokers import get_broker
 from django_q.conf import Conf, logger, psutil, get_ppid, error_reporter
 from django_q.models import Task, Success, Schedule
@@ -172,7 +173,7 @@ class Sentinel(object):
         else:
             self.pool.remove(process)
             self.spawn_worker()
-            if self.timeout and int(process.timer.value) == 0:
+            if process.timer.value == 0:
                 # only need to terminate on timeout, otherwise we risk destabilizing the queues
                 process.terminate()
                 logger.warn(_("reincarnated worker {} after timeout").format(process.name))
@@ -209,13 +210,14 @@ class Sentinel(object):
         while not self.stop_event.is_set() or not counter:
             # Check Workers
             for p in self.pool:
-                # Are you alive?
-                if not p.is_alive() or (self.timeout and p.timer.value == 0):
-                    self.reincarnate(p)
-                    continue
-                # Decrement timer if work is being done
-                if self.timeout and p.timer.value > 0:
-                    p.timer.value -= cycle
+                with p.timer.get_lock():
+                    # Are you alive?
+                    if not p.is_alive() or p.timer.value == 0:
+                        self.reincarnate(p)
+                        continue
+                    # Decrement timer if work is being done
+                    if p.timer.value > 0:
+                        p.timer.value -= cycle
             # Check Monitor
             if not self.monitor.is_alive():
                 self.reincarnate(self.monitor)
@@ -347,6 +349,8 @@ def worker(task_queue, result_queue, timer, timeout=Conf.TIMEOUT):
     name = current_process().name
     logger.info(_('{} ready for work at {}').format(name, current_process().pid))
     task_count = 0
+    if timeout is None:
+        timeout = -1
     # Start reading the task queue
     for task in iter(task_queue.get, 'STOP'):
         result = None
@@ -368,7 +372,7 @@ def worker(task_queue, result_queue, timer, timeout=Conf.TIMEOUT):
         # We're still going
         if not result:
             db.close_old_connections()
-            timer_value = task['kwargs'].pop('timeout', timeout or 0)
+            timer_value = task['kwargs'].pop('timeout', timeout)
             # signal execution
             pre_execute.send(sender="django_q", func=f, task=task)
             # execute the payload
@@ -380,16 +384,17 @@ def worker(task_queue, result_queue, timer, timeout=Conf.TIMEOUT):
                 result = ('{} : {}'.format(e, traceback.format_exc()), False)
                 if error_reporter:
                     error_reporter.report()
-        # Process result
-        task['result'] = result[0]
-        task['success'] = result[1]
-        task['stopped'] = timezone.now()
-        result_queue.put(task)
-        timer.value = -1  # Idle
-        # Recycle
-        if task_count == Conf.RECYCLE:
-            timer.value = -2  # Recycled
-            break
+        with timer.get_lock():
+            # Process result
+            task['result'] = result[0]
+            task['success'] = result[1]
+            task['stopped'] = timezone.now()
+            result_queue.put(task)
+            timer.value = -1  # Idle
+            # Recycle
+            if task_count == Conf.RECYCLE:
+                timer.value = -2  # Recycled
+                break
     logger.info(_('{} stopped doing work').format(name))
 
 
@@ -402,7 +407,7 @@ def save_task(task, broker):
         return
     # enqueues next in a chain
     if task.get('chain', None):
-        tasks.async_chain(task['chain'], group=task['group'], cached=task['cached'], sync=task['sync'], broker=broker)
+        django_q.tasks.async_chain(task['chain'], group=task['group'], cached=task['cached'], sync=task['sync'], broker=broker)
     # SAVE LIMIT > 0: Prune database, SAVE_LIMIT 0: No pruning
     db.close_old_connections()
     try:
@@ -470,7 +475,7 @@ def save_cached(task, broker):
             broker.cache.set(group_key, group_list, timeout)
             # async_task next in a chain
             if task.get('chain', None):
-                tasks.async_chain(task['chain'], group=group, cached=task['cached'], sync=task['sync'], broker=broker)
+                django_q.tasks.async_chain(task['chain'], group=group, cached=task['cached'], sync=task['sync'], broker=broker)
         # save the task
         broker.cache.set(task_key,
                          SignedPackage.dumps(task),
@@ -510,28 +515,30 @@ def scheduler(broker=None):
                 next_run = arrow.get(s.next_run)
                 while True:
                     if s.schedule_type == s.MINUTES:
-                        next_run = next_run.replace(minutes=+(s.minutes or 1))
+                        next_run = next_run.shift(minutes=+(s.minutes or 1))
                     elif s.schedule_type == s.HOURLY:
-                        next_run = next_run.replace(hours=+1)
+                        next_run = next_run.shift(hours=+1)
                     elif s.schedule_type == s.DAILY:
-                        next_run = next_run.replace(days=+1)
+                        next_run = next_run.shift(days=+1)
                     elif s.schedule_type == s.WEEKLY:
-                        next_run = next_run.replace(weeks=+1)
+                        next_run = next_run.shift(weeks=+1)
                     elif s.schedule_type == s.MONTHLY:
-                        next_run = next_run.replace(months=+1)
+                        next_run = next_run.shift(months=+1)
                     elif s.schedule_type == s.QUARTERLY:
-                        next_run = next_run.replace(months=+3)
+                        next_run = next_run.shift(months=+3)
                     elif s.schedule_type == s.YEARLY:
-                        next_run = next_run.replace(years=+1)
+                        next_run = next_run.shift(years=+1)
                     if Conf.CATCH_UP or next_run > arrow.utcnow():
                         break
-                s.next_run = next_run.datetime
+                # arrow always returns a tz aware datetime, and we don't want
+                # this when we explicitly configured django with USE_TZ=False
+                s.next_run = next_run.datetime if settings.USE_TZ else next_run.datetime.replace(tzinfo=None)
                 s.repeats += -1
             # send it to the cluster
             q_options['broker'] = broker
             q_options['group'] = q_options.get('group', s.name or s.id)
             kwargs['q_options'] = q_options
-            s.task = tasks.async_task(s.func, *args, **kwargs)
+            s.task = django_q.tasks.async_task(s.func, *args, **kwargs)
             # log it
             if not s.task:
                 logger.error(
